require('../../modules/es6.math.acosh');
require('../../modules/es6.math.asinh');
require('../../modules/es6.math.atanh');
require('../../modules/es6.math.cbrt');
require('../../modules/es6.math.clz32');
require('../../modules/es6.math.cosh');
require('../../modules/es6.math.expm1');
require('../../modules/es6.math.fround');
require('../../modules/es6.math.hypot');
require('../../modules/es6.math.imul');
require('../../modules/es6.math.log10');
require('../../modules/es6.math.log1p');
require('../../modules/es6.math.log2');
require('../../modules/es6.math.sign');
require('../../modules/es6.math.sinh');
require('../../modules/es6.math.tanh');
require('../../modules/es6.math.trunc');
require('../../modules/es7.math.clamp');
require('../../modules/es7.math.deg-per-rad');
require('../../modules/es7.math.degrees');
require('../../modules/es7.math.fscale');
require('../../modules/es7.math.iaddh');
require('../../modules/es7.math.isubh');
require('../../modules/es7.math.imulh');
require('../../modules/es7.math.rad-per-deg');
require('../../modules/es7.math.radians');
require('../../modules/es7.math.scale');
require('../../modules/es7.math.umulh');
require('../../modules/es7.math.signbit');
module.exports = require('../../modules/_core').Math;
