require('../../modules/es6.regexp.constructor');
require('../../modules/es6.regexp.to-string');
require('../../modules/es6.regexp.flags');
require('../../modules/es6.regexp.match');
require('../../modules/es6.regexp.replace');
require('../../modules/es6.regexp.search');
require('../../modules/es6.regexp.split');
require('../../modules/core.regexp.escape');
module.exports = require('../../modules/_core').RegExp;
