import { Subscriber } from '../Subscriber';
import { Subject } from '../Subject';
/**
 * Determines whether the ErrorObserver is closed or stopped or has a
 * destination that is closed or stopped - in which case errors will
 * need to be reported via a different mechanism.
 * @param observer the observer
 */
export declare function canReportError(observer: Subscriber<any> | Subject<any>): boolean;
