require('../../modules/es6.reflect.apply');
require('../../modules/es6.reflect.construct');
require('../../modules/es6.reflect.define-property');
require('../../modules/es6.reflect.delete-property');
require('../../modules/es6.reflect.enumerate');
require('../../modules/es6.reflect.get');
require('../../modules/es6.reflect.get-own-property-descriptor');
require('../../modules/es6.reflect.get-prototype-of');
require('../../modules/es6.reflect.has');
require('../../modules/es6.reflect.is-extensible');
require('../../modules/es6.reflect.own-keys');
require('../../modules/es6.reflect.prevent-extensions');
require('../../modules/es6.reflect.set');
require('../../modules/es6.reflect.set-prototype-of');
require('../../modules/es7.reflect.define-metadata');
require('../../modules/es7.reflect.delete-metadata');
require('../../modules/es7.reflect.get-metadata');
require('../../modules/es7.reflect.get-metadata-keys');
require('../../modules/es7.reflect.get-own-metadata');
require('../../modules/es7.reflect.get-own-metadata-keys');
require('../../modules/es7.reflect.has-metadata');
require('../../modules/es7.reflect.has-own-metadata');
require('../../modules/es7.reflect.metadata');
module.exports = require('../../modules/_core').Reflect;
