export interface TimeoutError extends Error {
}
export interface TimeoutErrorCtor {
    new (): TimeoutError;
}
/**
 * An error thrown when duetime elapses.
 *
 * @see {@link timeout}
 *
 * @class TimeoutError
 */
export declare const TimeoutError: TimeoutErrorCtor;
