require('../modules/es7.string.at');
require('../modules/es7.map.to-json');
require('../modules/es7.set.to-json');
require('../modules/es7.error.is-error');
require('../modules/es7.math.iaddh');
require('../modules/es7.math.isubh');
require('../modules/es7.math.imulh');
require('../modules/es7.math.umulh');
require('../modules/es7.asap');
module.exports = require('./1');
