import { OperatorFunction, ObservableInput } from '../types';
export declare function concatAll<T>(): OperatorFunction<ObservableInput<T>, T>;
export declare function concatAll<R>(): OperatorFunction<any, R>;
