require('../modules/es7.map.of');
require('../modules/es7.set.of');
require('../modules/es7.weak-map.of');
require('../modules/es7.weak-set.of');
require('../modules/es7.map.from');
require('../modules/es7.set.from');
require('../modules/es7.weak-map.from');
require('../modules/es7.weak-set.from');
require('../modules/es7.math.clamp');
require('../modules/es7.math.deg-per-rad');
require('../modules/es7.math.degrees');
require('../modules/es7.math.fscale');
require('../modules/es7.math.rad-per-deg');
require('../modules/es7.math.radians');
require('../modules/es7.math.scale');
require('../modules/es7.math.signbit');
require('../modules/es7.promise.try');
require('../modules/es7.string.match-all');
require('../modules/es7.symbol.observable');
require('../modules/es7.observable');
require('../modules/es7.array.flat-map');
require('../modules/es7.array.flatten');
module.exports = require('./2');
