import { ObservableInput, OperatorFunction } from '../types';
export declare function exhaust<T>(): OperatorFunction<ObservableInput<T>, T>;
export declare function exhaust<R>(): OperatorFunction<any, R>;
